/**
* Programa Tabuleiro:
*	Jogo desenvolvido para o ensino de desenvolvimento de jogos
*	de computadores, utilizando-se de uma matriz de elementos
*	para facilitar a interacao entre os mesmos.
*
*	Autor
*		Saulo Popov Zambiasi
*
*	Ultima alteracao
*		20 de Outubro de 2006.
*/

#include <stdio.h>
#include <allegro.h>

#define MAX_X 800
#define MAX_Y 600
#define VMODE GFX_AUTODETECT_WINDOWED
#define TEMPO_LIMITE 1

// Direcao

#define PARADO		0
#define CIMA		128
#define BAIXO		256
#define ESQUERDA	32
#define DIREITA		-32

// Tipos de elementos

#define PAREDE		1
#define MONSTRO		2
#define AVATAR		3

#define FALSO		0
#define VERDADEIRO	1

class Elemento {
public:
	int tipo;
	int direcao;
	int andando;
	BITMAP *bmp0;
	BITMAP *bmp1;
};

Elemento *tabuleiro[16][12];
BITMAP *buffer;
BITMAP *chao;
int passo;
int sair;
int tempo;

void aumenta_tempo (void) {
      tempo++;
}
END_OF_FUNCTION(aumenta_tempo);

void executar(){
	int x, y, aux, direcao;
	
	// Le informacoes do teclado para o avatar
	if (key[KEY_ESC]) sair = 1;
	else if (key[KEY_UP]) direcao = CIMA;
	else if (key[KEY_DOWN]) direcao = BAIXO;
	else if (key[KEY_LEFT]) direcao = ESQUERDA;
	else if (key[KEY_RIGHT]) direcao = DIREITA;
	
	// Executa os elementos do tabuleiro
	for (x = 0; x < 16; x++) {
		for (y = 0; y < 12; y++) {
			if (tabuleiro[x][y] != NULL) {
				if (passo == 0) {
					if (!tabuleiro[x][y]->andando) {
						// Verifica o novo movimento dos elementos
						switch (tabuleiro[x][y]->tipo) {
							case MONSTRO:
								//mudar direcao aleatoriamente
								aux = (int)(rand() / (((double)RAND_MAX + 1)/ 4)) + 1;
								if (aux == 1) tabuleiro[x][y]->direcao = CIMA;
								else if (aux == 2) tabuleiro[x][y]->direcao = BAIXO;
								else if (aux == 3) tabuleiro[x][y]->direcao = ESQUERDA;
								else if (aux == 4) tabuleiro[x][y]->direcao = DIREITA;
								break;
							case AVATAR:
								tabuleiro[x][y]->direcao = direcao;
								break;
						}
						// Movimenta
						switch (tabuleiro[x][y]->direcao) {
							case CIMA:
								if (y > 0) {
									if (tabuleiro[x][y-1] == NULL) {
										tabuleiro[x][y]->andando = VERDADEIRO;
										tabuleiro[x][y-1] = tabuleiro[x][y];
										tabuleiro[x][y] = NULL;
									}
								}
								break;
							case BAIXO:
								if (y < 11) {
									if (tabuleiro[x][y+1] == NULL) {
										tabuleiro[x][y]->andando = VERDADEIRO;
										tabuleiro[x][y+1] = tabuleiro[x][y];
										tabuleiro[x][y] = NULL;
									}
								}
								break;
							case ESQUERDA:
								if (x > 0) {
									if (tabuleiro[x-1][y] == NULL) {
										tabuleiro[x][y]->andando = VERDADEIRO;
										tabuleiro[x-1][y] = tabuleiro[x][y];
										tabuleiro[x][y] = NULL;
									}
								}
								break;
							case DIREITA:
								if (x < 15) {
									if (tabuleiro[x+1][y] == NULL) {
										tabuleiro[x][y]->andando = VERDADEIRO;
										tabuleiro[x+1][y] = tabuleiro[x][y];
										tabuleiro[x][y] = NULL;
									}
								}
								break;
						}
					}
				} else {
					if (tabuleiro[x][y] != NULL) {
						tabuleiro[x][y]->andando = FALSO;
					}
				}
			}
		}
	}
	if (passo == 0) passo = 1;
	else passo = 0;
	clear_keybuf();
}

void desenhar() {
	int gira = 0;
	
	clear (buffer);
	blit(chao, buffer, 0, 0, 0, 0, MAX_X, MAX_Y);
	int x, y;
	for (x = 0; x < 16; x++) {
		for (y = 0; y < 12; y++) {
			if (tabuleiro[x][y] != NULL) {
				BITMAP *bmp;
				int posx = x * 50;
				int posy = y * 50;
				if (tabuleiro[x][y]->andando) {
					switch (tabuleiro[x][y]->direcao) {
						case CIMA: posy += 25; break;
						case BAIXO: posy -= 25; break;
						case ESQUERDA: posx += 25; break;
						case DIREITA: posx -= 25; break;
					}
				}
				if (tabuleiro[x][y]->andando) bmp = tabuleiro[x][y]->bmp0;
				else bmp = tabuleiro[x][y]->bmp1;
				rotate_sprite(buffer, bmp, posx, posy, itofix(tabuleiro[x][y]->direcao));
			}
		}
	}
	blit(buffer, screen, 0, 0, 0, 0, MAX_X, MAX_Y);
}

void constroi_ambiente() {
    char str[18];
    FILE *fp;
    if ((fp = fopen("tabuleiro.cfg", "r")) == NULL) {
        printf ("Arquivo nao pode ser aberto\n");
        exit(1);
    }
    Elemento *e;
    int x = 0;
    int y = 0;
    while (!feof(fp) && (y < 12)) {
        fgets(str, 18, fp);
        printf ("%c\n", str);
        for (x = 0; x < 16; x++) {
            if (str[x] == '0') {
                tabuleiro[x][y] = NULL;
            } else if (str[x] == '1') {
                e = new Elemento();
                e->tipo = PAREDE;
                e->bmp0 = load_bitmap("parede.bmp", NULL);
                e->bmp1 = e->bmp0;
                e->andando = FALSO;
				e->direcao = PARADO;
                tabuleiro[x][y] = e;
            } else if (str[x] == '2') {
                e = new Elemento();
                e->tipo = MONSTRO;
                e->bmp0 = load_bitmap("monstro0.bmp", NULL);
                e->bmp1 = load_bitmap("monstro1.bmp", NULL);
				e->direcao = PARADO;
                e->andando = FALSO;
                tabuleiro[x][y] = e;
            } else if (str[x] == '3') {
                e = new Elemento();
                e->tipo = AVATAR;
                e->bmp0 = load_bitmap("avatar0.bmp", NULL);
                e->bmp1 = load_bitmap("avatar1.bmp", NULL);
				e->direcao = PARADO;
                e->andando = FALSO;
                tabuleiro[x][y] = e;
            } else tabuleiro[x][y] = NULL;
        }
        y++;
    }
    fclose(fp);
}

int main (){
	allegro_init();
	install_keyboard();
	install_timer();
	set_color_depth(16);
	set_gfx_mode(VMODE, MAX_X, MAX_Y, 0, 0);
	buffer = create_bitmap(MAX_X, MAX_Y);
	chao = load_bitmap("chao.bmp", NULL);

	tempo = 0;
	install_int(aumenta_tempo, 1000);	

    constroi_ambiente();
    passo = 0;

	LOCK_VARIABLE(tempo);
	LOCK_FUNCTION(incrementa_tempo);	
	while (!sair){
        executar();
		desenhar();
		rest(100);
	}
    
    for (int x = 0; x < 16; x++) {
        for (int y = 0; y < 12; y++) {
            if (tabuleiro[x][y] != NULL)
                delete tabuleiro[x][y];
            }
    }
	destroy_bitmap(buffer);
	destroy_bitmap(chao);
	allegro_exit();
	return 0;
}
END_OF_MAIN();
